from PIL.Image import Image
import PIL
from .inpaint import (
    telea_inpaint,
)
import numpy as np
from typing import Union


class InpaintError(Exception):
    """Raised when inpainting failed for whatever reason."""


def telea(
    image: Union[Image, np.array],
    mask: Union[Image, np.array],
    radius: int = 5,
) -> Union[Image, np.array]:
    """Inpaint the image using the Telea algorithm

    Args:
        image: image to inpaint
        mask: image containing black and white mask for region to inpaint
        radius (optional): radius of near pixels that are considered for
                           inpainting. Defaults to 5.

    Raises:
        InpaintError: if something goes unrecoverably wrong.

    Returns:
        inpainted image
    """

    is_pil_image = isinstance(image, PIL.Image.Image)

    image_array = np.array(image) if is_pil_image else image
    mask_array = np.array(mask)[:, :, 0] if is_pil_image else mask

    original_image_type = image_array.dtype
    image_array = _convert_to_float(image_array)
    mask_array = _convert_to_float(mask_array)

    output: np.array = telea_inpaint(
        image_array,
        mask_array,
        radius,
    )

    if not np.issubdtype(original_image_type, np.floating):
        output *= np.iinfo(original_image_type).max

    if is_pil_image:
        return PIL.Image.fromarray(output.astype(original_image_type))

    return output


def _convert_to_float(image_array: np.array):
    """Convert

    Args:
        image_array: array to convert to float
    """

    if not np.issubdtype(image_array.dtype, np.floating):
        image_array = image_array.astype(np.float32, copy=False) / float(
            np.iinfo(
                image_array.dtype,
            ).max
        )

    return image_array



import pytest
from PIL import Image
import inpaint
import numpy as np


@pytest.mark.parametrize(("input_type"), [np.float32, np.uint32, np.int32])
def test_numpy_interface(input_type) -> None:
    """Make sure return type is correct, as telea inpaint supports both images and arrays

    Args:
        input_type:: type for array, should match return value
    """
    test_image = Image.open("./test/images/input/bird.png")
    test_array = np.asarray(test_image, dtype=np.float32) / 256.0
    test_array = np.asarray(test_image, dtype=input_type)

    output = inpaint.telea(test_array, test_array[:, :, 0])

    assert type(output) is type(test_array)



from pathlib import Path
import pytest
from PIL import Image
from PIL.PngImagePlugin import PngImageFile
import inpaint
import numpy as np


def compare_image_arrays(a: PngImageFile, b: PngImageFile, tolerance: float):
    """Simple comparison util for the images"""
    if a.width != b.width or a.height != b.height:
        raise ValueError("Images differ in dimensions.")

    a_array = np.asarray(a, dtype=np.float32) / 255.0
    b_array = np.asarray(b, dtype=np.float32) / 255.0

    close_elements = np.isclose(a_array, b_array, atol=tolerance)

    similar_count = np.sum(close_elements)
    total_elements = a_array.size

    percentage = (similar_count / total_elements) * 100

    return percentage


@pytest.mark.parametrize(
    ("input_image", "input_mask", "expected_image"),
    [
        (
            "./test/images/input/bird.png",
            "./test/images/mask/thin.png",
            "./test/images/expected/telea/bird_thin.png",
        ),
        (
            "./test/images/input/bird.png",
            "./test/images/mask/medium.png",
            "./test/images/expected/telea/bird_medium.png",
        ),
        (
            "./test/images/input/bird.png",
            "./test/images/mask/large.png",
            "./test/images/expected/telea/bird_large.png",
        ),
        (
            "./test/images/input/bird.png",
            "./test/images/mask/text.png",
            "./test/images/expected/telea/bird_text.png",
        ),
        (
            "./test/images/input/toad.png",
            "./test/images/mask/thin.png",
            "./test/images/expected/telea/toad_thin.png",
        ),
        (
            "./test/images/input/toad.png",
            "./test/images/mask/medium.png",
            "./test/images/expected/telea/toad_medium.png",
        ),
        (
            "./test/images/input/toad.png",
            "./test/images/mask/text.png",
            "./test/images/expected/telea/toad_text.png",
        ),
    ],
)
def test_images(input_image: Path, input_mask: Path, expected_image: Path) -> None:
    """Test images as also tested in Rust package, to confirm the bridge is working as expected"""
    image = Image.open(input_image)
    mask = Image.open(input_mask)
    expected = Image.open(expected_image)

    output = inpaint.telea(image, mask)

    result = compare_image_arrays(output, expected, 0.005)
    assert result == 100.0



import time
import inpaint
from PIL import Image

def main():
    image = Image.open("../../test/images/baked/toad.png")
    mask = Image.open("../../test/images/mask/text.png")

    start_time = time.time()
    output = inpaint.telea(image, mask, 5)
    end_time = time.time()

    output.save("./output.png")

    elapsed_time = end_time - start_time
    print(f"Inpainting finished in {elapsed_time:.2f} second(s)")

if __name__ == "__main__":
    main()
